require "rack/haproxy_status/version"
require "json"

module Rack
  module HaproxyStatus
    class Endpoint
      InvalidStatus = Class.new(StandardError)

      VALID_STATES = %w(on off)

      def initialize(path:, io: ::File, extra_checks: [])
        @path = path
        @io = io
        @extra_checks = extra_checks
      end

      def call(env)
        balancer_member = balancer_member?
        extra_checks_all_successful = extra_checks_all_successful?
        http_code = balancer_member && extra_checks_all_successful ? 200 : 503
        status = extra_checks_all_successful ? 'ok' : 'error'

        [
          http_code,
          {'Content-Type' => 'application/json'},
          [JSON.generate(status: status, member: balancer_member?)]
        ]
      rescue InvalidStatus
        [500, {'Content-Type' => 'application/json'}, ['{"status": "unknown status"}']]
      end

      private

      def balancer_member?
        state = @io.read(@path).strip
        raise InvalidStatus, "Invalid state: #{state}" unless VALID_STATES.include?(state)
        state == "on"
      end

      def extra_checks_all_successful?
        @extra_checks.map { |check| check.call() }.all?
      end
    end
  end
end
